#!/usr/bin/env perl
#
# NAME:
#	find_aliases.pl - find all the names that map to a given address.
#
# SYNOPSIS:
#	find_aliases.pl [options] "ip" ...
#
# DESCRIPTION:
#	This tool finds all the names by which "ip" is known.
#	If "ip" is a domain name, it is first resolved to an IP
#	address.
#
#	Output lists the IP address, followed by the canonical name
#	(from the PTR record) and then all the other names.
#	
# SEE ALSO:
#	getdata.pl(1), updrev.pl(1).
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

$RCSid = '$Id: find_aliases.pl,v 1.3 2008/01/26 04:25:46 sjg Exp $'; #'

#
#	@(#)Copyright (c) 1997-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#

# much of this is from uprev.pl

if ($0 =~ m,^(.*)/([^/]+)$,) {
  $Mydir = $1;
  $Myname = $2;
} else {
  $Mydir = '.';
  $Myname = $0;
}
$Myname =~ s/\.pl//;

push(@INC, $Mydir);
$ENV{'PATH'} = "$Mydir:$ENV{'PATH'}";

$SkipRC = 1;
require "$Mydir/funcs.pl";

&source('.dnsrc.pl');
&source(1, '.updrevrc.pl','etc',$Mydir,'.');
&source(1, ".${Myname}rc.pl",'etc',$Mydir,'.');

require "$Mydir/getopts.pl";

&Getopts('d:D:');

&main;
exit 0;

sub main {
  &loadData();

  foreach $ip (@ARGV) {
    if ($ip !~ m/^\d+(\.\d+)+/o) {
      $ip = &findA($ip);
    }
    @names = &findNames($ip);
    $ptr = $PTR{$ip};
    %u = ();
    foreach $n (@names) {
      $u{$n} = 1;
    }
    delete $u{$ptr};
    print "$ip\t$ptr\t", join(' ', sort(keys(%u))), "\n";
  }
}

sub revA {
  local(@a) = @_;
  local($i, $j, @r);

  for ($i = 0, $j = $#a; $i < scalar(@a); --$j, ++$i) {
    $r[$i] = $a[$j];
  }
  @r;
}

sub loadData {
  if ($opt_D ne '') {
    open(STDIN, "< $opt_D") || die "Cannot read $opt_D, stopped";
  }
  %A = ();
  %PTR = ();
  %Names = ();
  
  while (<STDIN>) {
    chop;
    next if (m/localhost/);
    next if (m/\s1.0.0.127.in-addr/); # some folk call it something else!
    
    s/\./ /;
    ($type,$key,$origin,$data,$xtra) = split(/\s+/, $_, 5);

    if ($type eq 'PTR') {
      $a = "$key.$origin";
      $a =~ s/\.in-addr.*//;
      @a = split(/\./, $a);
      if (($n = scalar(@a)) > 4) {
	# must be a CIDR PTR we want the 1st N - 4 
	# and the last N - (4 - (N - 4)) octets.
	$x = $n - 5;
	$y = $n - (3 - $x);
	@a = ($a[0 .. $x], @a[$y .. $#a]);
      }
      $a = join('.', &revA(@a));
      $PTR{$a} = $data;
      $A{$data} = $a;
      $Names{$a} = $data;
      next;
    }
    if ($type eq 'A') {
      next if ($data eq '127.0.0.1'); # !
      $a = $data;
      $fqdn = "$key.$origin";
      $A{$fqdn} = $a;
      $Names{$a} .= ",$fqdn";
      next;
    }
    if ($type eq 'CNAME') {
      $fqdn = "$key.$origin";
      $Names{$data} .= ",$fqdn";
      $CNAME{$fqdn} = $data;
      next;
    }
  }
}

sub findNames {
  local($ip) = @_;
  local(@names) = ();
  local($name);
  
  foreach $name (split(/,/, $Names{$ip})) {
    next if ($name eq '');
    push(@names, $name);
    if ($Names{$name} ne '') {
      push(@names, &findNames($name));
    }
  }
  @names;
}

sub findA {
  local($name) = @_;

  while ($CNAME{$name} ne '') {
    $name = $CNAME{$name};
  }
  $A{$name};
}
